#######################################
#### get model with specified size ####
#######################################
get.size.plcosso <- function(obj, size){
  whs <- (which(apply(obj$phi,2, function(x){sum(x>0)})==size))
  if (length(whs)){
    wh <- max(whs)
    obj$btheta <- obj$theta[,wh]
    obj$ba <- obj$a[,wh]
    obj$bphi <- obj$phi[,wh]
    obj$ind <- (obj$bphi>0)
    obj$whbestG <- wh
  } else {
    obj <- NA
  }
  return(obj)
}


#####################
#### bivariate 1 ####
#####################
# SNR=2.2

getmyfourierbasismatrix <- function(x, maxlim, nterms, adj, shift=0){
  m <- length(x)
  out <- matrix(nr=m, nc=nterms)
  for (j in (1:nterms)){
    out[,j] <- adj * sin(pi*j*x/maxlim + shift)
  }
  return(out)
}


sim.biexample1 <- function(n, N, J, v=sqrt(0.2), sigma=1){
 # n: sample sizes
 # N: number of time points
 # J: number of eigenfunctions
 # v: sd of error (functional predictor)
 # sigma: sd of error (response)

  ## functional predictor data
  mu1 <- function(t) {t + sin(t)}  # true mean function t \in [0,10]
  mu2 <- function(t) {t + cos(t)}  # true mean function t \in [0,10]
  lam <- 45.25 * 0.64^(0:(J-1))   # eigenvalues

  xi0 <- matrix(rnorm(n*J, sd=rep(sqrt(lam), each=n)), nr=n, nc=J)  # FPC scores
  tt <- matrix(seq(0, 10, len=N), nr=n, nc=N, byrow=T)             # time points
  X01 <- matrix(nr=n, nc=N)   # functional predictors with no measurement errors
  X02 <- matrix(nr=n, nc=N)   # functional predictors with no measurement errors
  for (i in (1:n)){
    bmat1 <- getmyfourierbasismatrix(tt[i,], maxlim=10, nterms=J, adj=sqrt(1/10),
                                     shift=pi/4)
    bmat2 <- getmyfourierbasismatrix(tt[i,], maxlim=10, nterms=J, adj=sqrt(1/10),
                                     shift=3*pi/4)
    X01[i,] <- mu1(tt[i,]) + bmat1 %*% xi0[i,]
    X02[i,] <- mu2(tt[i,]) + bmat2 %*% xi0[i,]
  }
  X1 <- X01 + matrix(rnorm(n*N, sd=v), nr=n, nc=N)
  X2 <- X02 + matrix(rnorm(n*N, sd=v), nr=n, nc=N)

  ## cosso regression data
  ### linear part
  theta0 <- c(0,0)
  p <- length(theta0)
  U <- matrix(runif(n*p), nr=n, nc=p)

  ### functional part
  ## only depends on the first, second and the fourth zeta
  b0 <- 1.4
  f1 <- function(z) {3*z - 3/2}
  f2 <- function(z) {sin(2*pi*(z - 1/2))}
  f4 <- function(z) {8*(z - 1/3)^2 - 8/9}

  zeta0 <- pnorm(xi0, sd=rep(sqrt(lam), each=n)) # zeta
  y0 <- as.vector(b0 + U%*%theta0 + f1(zeta0[,1]) + f2(zeta0[,2]) + f4(zeta0[,4]))
  y <- y0 + rnorm(n, sd=sigma)

  return(list(tt=tt, X1=X1, X2=X2, U=U, y=y, y0=y0, zeta0=zeta0))
}


sim.biexample2 <- function(n, N, J, v=sqrt(0.2), sigma=1){
 # n: sample sizes
 # N: number of time points
 # J: number of eigenfunctions
 # v: sd of error (functional predictor)
 # sigma: sd of error (response)

  ## functional predictor data
  mu1 <- function(t) {t + sin(t)}  # true mean function t \in [0,10]
  mu2 <- function(t) {t + cos(t)}  # true mean function t \in [0,10]
  lam <- 45.25 * 0.64^(0:(J-1))   # eigenvalues

  xi0 <- matrix(rnorm(n*J, sd=rep(sqrt(lam), each=n)), nr=n, nc=J)  # FPC scores
  tt <- matrix(seq(0, 10, len=N), nr=n, nc=N, byrow=T)             # time points
  X01 <- matrix(nr=n, nc=N)   # functional predictors with no measurement errors
  X02 <- matrix(nr=n, nc=N)   # functional predictors with no measurement errors
  for (i in (1:n)){
    bmat1 <- getmyfourierbasismatrix(tt[i,], maxlim=10, nterms=J, adj=sqrt(1/10),
                                     shift=pi/4)
    bmat2 <- getmyfourierbasismatrix(tt[i,], maxlim=10, nterms=J, adj=sqrt(1/10),
                                     shift=3*pi/4)
    X01[i,] <- mu1(tt[i,]) + bmat1 %*% xi0[i,]
    X02[i,] <- mu2(tt[i,]) + bmat2 %*% xi0[i,]
  }
  X1 <- X01 + matrix(rnorm(n*N, sd=v), nr=n, nc=N)
  X2 <- X02 + matrix(rnorm(n*N, sd=v), nr=n, nc=N)

  ## cosso regression data
  ### linear part
  theta0 <- c(3,-4)
  p <- length(theta0)
  U <- matrix(runif(n*p), nr=n, nc=p)

  ### functional part
  ## only depends on the first, second and the fourth zeta
  b0 <- 1.4
  f1 <- function(z) {3*z - 3/2}
  f2 <- function(z) {sin(2*pi*(z - 1/2))}
  f4 <- function(z) {8*(z - 1/3)^2 - 8/9}

  zeta0 <- pnorm(xi0, sd=rep(sqrt(lam), each=n)) # zeta
  y0 <- as.vector(b0 + U%*%theta0 + f1(zeta0[,1]) + f2(zeta0[,2]) + f4(zeta0[,4]))
  y <- y0 + rnorm(n, sd=sigma)

  return(list(tt=tt, X1=X1, X2=X2, U=U, y=y, y0=y0, zeta0=zeta0))
}


sim.biexample3 <- function(n, N, J, v=sqrt(0.2), sigma=1){
 # n: sample sizes
 # N: number of time points
 # J: number of eigenfunctions
 # v: sd of error (functional predictor)
 # sigma: sd of error (response)

  ## functional predictor data
  mu1 <- function(t) {t + sin(t)}  # true mean function t \in [0,10]
  mu2 <- function(t) {t + cos(t)}  # true mean function t \in [0,10]
  #lam <- 45.25 * 0.64^(0:(J-1))   # eigenvalues
  lam <- 45.25 * 0.64 * (1:J)^(-2)   # eigenvalues

  xi0 <- matrix(rnorm(n*J, sd=rep(sqrt(lam), each=n)), nr=n, nc=J)  # FPC scores
  tt <- matrix(seq(0, 10, len=N), nr=n, nc=N, byrow=T)             # time points
  X01 <- matrix(nr=n, nc=N)   # functional predictors with no measurement errors
  X02 <- matrix(nr=n, nc=N)   # functional predictors with no measurement errors
  for (i in (1:n)){
    bmat1 <- getmyfourierbasismatrix(tt[i,], maxlim=10, nterms=J, adj=sqrt(1/10),
                                     shift=pi/4)
    bmat2 <- getmyfourierbasismatrix(tt[i,], maxlim=10, nterms=J, adj=sqrt(1/10),
                                     shift=3*pi/4)
    X01[i,] <- mu1(tt[i,]) + bmat1 %*% xi0[i,]
    X02[i,] <- mu2(tt[i,]) + bmat2 %*% xi0[i,]
  }
  X1 <- X01 + matrix(rnorm(n*N, sd=v), nr=n, nc=N)
  X2 <- X02 + matrix(rnorm(n*N, sd=v), nr=n, nc=N)

  ## cosso regression data
  ### linear part
  theta0 <- c(0,0)
  p <- length(theta0)
  U <- matrix(runif(n*p), nr=n, nc=p)

  ### functional part
  ## only depends on the first, second and the fourth zeta
  b0 <- 1.4
  f1 <- function(z) {3*z - 3/2}
  f2 <- function(z) {sin(2*pi*(z - 1/2))}
  f4 <- function(z) {8*(z - 1/3)^2 - 8/9}

  zeta0 <- pnorm(xi0, sd=rep(sqrt(lam), each=n)) # zeta
  y0 <- as.vector(b0 + U%*%theta0 + f1(zeta0[,1]) + f2(zeta0[,2]) + f4(zeta0[,4]))
  y <- y0 + rnorm(n, sd=sigma)

  return(list(tt=tt, X1=X1, X2=X2, U=U, y=y, y0=y0, zeta0=zeta0))
}


sim.biexample4 <- function(n, N, J, v=sqrt(0.2), sigma=1){
 # n: sample sizes
 # N: number of time points
 # J: number of eigenfunctions
 # v: sd of error (functional predictor)
 # sigma: sd of error (response)

  ## functional predictor data
  mu1 <- function(t) {t + sin(t)}  # true mean function t \in [0,10]
  mu2 <- function(t) {t + cos(t)}  # true mean function t \in [0,10]
  #lam <- 45.25 * 0.64^(0:(J-1))   # eigenvalues
  lam <- 45.25 * 0.64 * (1:J)^(-2)   # eigenvalues

  xi0 <- matrix(rnorm(n*J, sd=rep(sqrt(lam), each=n)), nr=n, nc=J)  # FPC scores
  tt <- matrix(seq(0, 10, len=N), nr=n, nc=N, byrow=T)             # time points
  X01 <- matrix(nr=n, nc=N)   # functional predictors with no measurement errors
  X02 <- matrix(nr=n, nc=N)   # functional predictors with no measurement errors
  for (i in (1:n)){
    bmat1 <- getmyfourierbasismatrix(tt[i,], maxlim=10, nterms=J, adj=sqrt(1/10),
                                     shift=pi/4)
    bmat2 <- getmyfourierbasismatrix(tt[i,], maxlim=10, nterms=J, adj=sqrt(1/10),
                                     shift=3*pi/4)
    X01[i,] <- mu1(tt[i,]) + bmat1 %*% xi0[i,]
    X02[i,] <- mu2(tt[i,]) + bmat2 %*% xi0[i,]
  }
  X1 <- X01 + matrix(rnorm(n*N, sd=v), nr=n, nc=N)
  X2 <- X02 + matrix(rnorm(n*N, sd=v), nr=n, nc=N)

  ## cosso regression data
  ### linear part
  theta0 <- c(3,-4)
  p <- length(theta0)
  U <- matrix(runif(n*p), nr=n, nc=p)

  ### functional part
  ## only depends on the first, second and the fourth zeta
  b0 <- 1.4
  f1 <- function(z) {3*z - 3/2}
  f2 <- function(z) {sin(2*pi*(z - 1/2))}
  f4 <- function(z) {8*(z - 1/3)^2 - 8/9}

  zeta0 <- pnorm(xi0, sd=rep(sqrt(lam), each=n)) # zeta
  y0 <- as.vector(b0 + U%*%theta0 + f1(zeta0[,1]) + f2(zeta0[,2]) + f4(zeta0[,4]))
  y <- y0 + rnorm(n, sd=sigma)

  return(list(tt=tt, X1=X1, X2=X2, U=U, y=y, y0=y0, zeta0=zeta0))
}
